#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class RODC : Indicator
	{
		private double          _point, LamUp, LamDn, MULT, tmpMin, tmpMax, xext, xcls;
		private int             _digits, jext, jj, limit, nUD, NumUD, segstr, Strt;
		private int[]           AryUD = new int[150];
		private bool            modeUp;
		private Series<double>  ExtZigzagBuffer;

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description 		= @"Rate of directional change (RODC) Indicator as published in the March 2024 Stocks and Commodities article titled ""Taming The Effects Of Whipsaw"" by Richard Poster, PhD.";
				Name 				= "RDOC";
				Calculate 			= Calculate.OnBarClose;
				DrawOnPricePanel 	= true;
				DisplayWarnings 	= true;
				BkData 				= 30;
				Lamda 				= 15;
				PerSmth 			= 3;

				AddPlot(new Stroke(Brushes.Blue, 2), PlotStyle.Line, "RDOC");
			}
			else if (State == State.Configure)
			{
				if (Instrument.MasterInstrument.InstrumentType == InstrumentType.Forex)
				{
					AddLine(Brushes.DarkOrange, 0, "Minimum");
					AddLine(Brushes.DarkOrange, 10, "Level1");
					AddLine(Brushes.DarkOrange, 20, "Level2");
					AddLine(Brushes.DarkOrange, 30, "Level3");
				}
			}
			else if (State == State.DataLoaded)
			{
				MULT    = 1.0d;

				ExtZigzagBuffer     = new Series<double>(this, MaximumBarsLookBack.Infinite);
				_point              = TickSize;
				_digits             = BitConverter.GetBytes(decimal.GetBits((decimal)TickSize)[3])[2];
                
				limit               = BkData + PerSmth + 1;
				
				if (_digits == 5 || _digits == 3)
				{
					MULT    = 10.0;
					LamUp   = Lamda;
					LamDn   = Lamda;
				}
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < limit)
				return;

			if (DisplayWarnings && Instrument.MasterInstrument.InstrumentType != InstrumentType.Forex)
			{
				Draw.TextFixed(this, "ROCD Error Text", "The RDOC Smoothed indicator is designed for use on forex instruments. Using it on other types of instruments will prevent the pre defined levels from being displayed.", TextPosition.BottomRight);
			}           

			nUD		= 1;
			modeUp	= true;

			AryUD	= new int[150];
			segstr	= CurrentBar - BkData + 1;
			Strt	= CurrentBar;

			AryUD[0]	= segstr;
			xext		= Close.GetValueAt(segstr - 1);
			jext		= segstr - 1;

			for (jj = segstr; jj <= Strt; jj++)
			{
				xcls = Close.GetValueAt(jj);
				if (!modeUp)
				{
					var a = (xcls - xext) / (MULT * _point);
					if (xext > xcls)
					{
						xext	= xcls;
						jext	= jj;
					}
					else if ((xcls - xext) / (MULT * _point) >= LamDn)
					{
						modeUp			= true;
						nUD				+= 1;
						AryUD[nUD - 1]	= jext;
						xext			= xcls;
						jext			= jj;
					}
				}
				else if (modeUp)
				{
					if (xext < xcls)
					{
						xext	= xcls;
						jext	= jj;
					}
					else if ((xcls - xext) / (MULT * _point) <= -LamUp)
					{
						modeUp			= false;
						nUD				+= 1;
						AryUD[nUD - 1]	= jext;
						xext			= xcls;
						jext			= jj;
					}
				}
			}

			NumUD = nUD;
			
			if (NumUD == 0)
				ExtZigzagBuffer[0] = 100.0d / BkData;
			else
				ExtZigzagBuffer[0] = 100.0d * NumUD / BkData;
			
			RDOC[0] = 0.0d;
			
			for (jj = 0; jj < PerSmth; jj++)
			{
				RDOC[0] += ExtZigzagBuffer.GetValueAt(CurrentBar - jj);
			}
			
			RDOC[0] = RDOC[0] / PerSmth;
		}

		public override void OnCalculateMinMax()
		{
			if (Instrument != null && Instrument.MasterInstrument.InstrumentType != InstrumentType.Forex)
			{
				base.OnCalculateMinMax();
				return;
			}
			
			tmpMin = double.MaxValue;
			tmpMax = double.MinValue;

			for (int index = ChartBars.FromIndex; index <= ChartBars.ToIndex; index++)
			{
				double plotValue = RDOC.GetValueAt(index);
				tmpMin = Math.Min(tmpMin, plotValue);
				tmpMax = Math.Max(tmpMax, plotValue);
			}

			MinValue = 0;
			MaxValue = tmpMax;
		}

		#region Properties
		[Display(Name = "Display Warnings", Description = "Show wanrings for non forex instruments", Order = 1, GroupName = "Parameters")]
		public bool DisplayWarnings
		{
		get; set;
		}

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name = "Window", Description = "Window", Order = 2, GroupName = "Parameters")]
		public int BkData
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, double.MaxValue)]
		[Display(Name = "ZigZag Threshold", Description = "Zigzag Threshold", Order = 3, GroupName = "Parameters")]
		public double Lamda
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name = "Smoothing Period", Description = "Smooth Period", Order = 4, GroupName = "Parameters")]
		public int PerSmth
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> RDOC
		{
			get { return Values[0]; }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private RODC[] cacheRODC;
		public RODC RODC(int bkData, double lamda, int perSmth)
		{
			return RODC(Input, bkData, lamda, perSmth);
		}

		public RODC RODC(ISeries<double> input, int bkData, double lamda, int perSmth)
		{
			if (cacheRODC != null)
				for (int idx = 0; idx < cacheRODC.Length; idx++)
					if (cacheRODC[idx] != null && cacheRODC[idx].BkData == bkData && cacheRODC[idx].Lamda == lamda && cacheRODC[idx].PerSmth == perSmth && cacheRODC[idx].EqualsInput(input))
						return cacheRODC[idx];
			return CacheIndicator<RODC>(new RODC(){ BkData = bkData, Lamda = lamda, PerSmth = perSmth }, input, ref cacheRODC);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.RODC RODC(int bkData, double lamda, int perSmth)
		{
			return indicator.RODC(Input, bkData, lamda, perSmth);
		}

		public Indicators.RODC RODC(ISeries<double> input , int bkData, double lamda, int perSmth)
		{
			return indicator.RODC(input, bkData, lamda, perSmth);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.RODC RODC(int bkData, double lamda, int perSmth)
		{
			return indicator.RODC(Input, bkData, lamda, perSmth);
		}

		public Indicators.RODC RODC(ISeries<double> input , int bkData, double lamda, int perSmth)
		{
			return indicator.RODC(input, bkData, lamda, perSmth);
		}
	}
}

#endregion
